<?php declare(strict_types=1);

namespace App\Entity;

use App\Repository\MenuItemRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\IdGenerator\UlidGenerator;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Uid\Ulid;

#[ORM\Entity(repositoryClass: MenuItemRepository::class)]
class MenuItem
{
    #[ORM\Id]
    #[ORM\GeneratedValue(strategy: 'CUSTOM')]
    #[ORM\Column(type: UlidType::NAME, unique: true)]
    #[ORM\CustomIdGenerator(class: UlidGenerator::class)]
    private Ulid $id;

    #[ORM\ManyToOne(inversedBy: 'menuItems')]
    #[ORM\JoinColumn(nullable: false)]
    private Vendor $vendor;

    #[ORM\Column]
    private int $price;

    #[ORM\OneToMany(
        mappedBy: 'menuItem',
        targetEntity: MenuItemAlias::class,
        orphanRemoval: true,
    )]
    private Collection $menuItemAliases;

    #[ORM\OneToMany(mappedBy: 'menuItem', targetEntity: ItemExtra::class, orphanRemoval: true)]
    private Collection $itemExtras;

    public function __construct()
    {
        $this->id = new Ulid;
        $this->menuItemAliases = new ArrayCollection;
        $this->itemExtras = new ArrayCollection();
    }

    public function getId(): Ulid
    {
        return $this->id;
    }

    public function getVendor(): Vendor
    {
        return $this->vendor;
    }

    public function setVendor(Vendor$vendor): static
    {
        $this->vendor = $vendor;
        return $this;
    }

    public function getPrice(): int|null
    {
        return $this->price;
    }

    public function setPrice(int $price): static
    {
        $this->price = $price;
        return $this;
    }

    /**
     * @return Collection<int, MenuItemAlias>
     */
    public function getMenuItemAliases(): Collection
    {
        return $this->menuItemAliases;
    }

    public function addMenuItemAlias(MenuItemAlias $menuItemAlias): static
    {
        if (! $this->menuItemAliases->contains($menuItemAlias)) {
            $this->menuItemAliases->add($menuItemAlias);
            $menuItemAlias->setMenuItem($this);
        }

        return $this;
    }

    public function removeMenuItemAlias(MenuItemAlias $menuItemAlias): static
    {
        // set the owning side to null (unless already changed)
        if (
            $this->menuItemAliases->removeElement($menuItemAlias)
            && $menuItemAlias->getMenuItem() === $this
        ) {
            $menuItemAlias->setMenuItem(null);
        }

        return $this;
    }
}
